@@ -25,7 +25,7 @@ import hashlib
 import datetime
 import socket
 import httplib
-import pprint
+import urllib
 
 import config
 import utils
@@ -51,7 +51,6 @@ def hook___import(cmd):
         data = json.loads(f.read())
         if data['type'] == 'cmd':
             run_remote(data['data'])
-    data = ''
     if len(cmd) > 1:
         for d in cmd[1:]:
             with open(d) as f:
@@ -60,29 +59,6 @@ def hook___import(cmd):
         _import(sys.stdin)
     sys.exit(0)
 
-def display_string(data):
-    print data
-
-def display_list(data):
-    for d in data:
-        display_string(d)
-
-def display_dict(data):
-    pp = pprint.PrettyPrinter(indent=4)
-    pp.pprint(data)
-
-def display(data):
-    display_handlers = {
-            basestring: display_string,
-            list: display_list,
-            dict: display_dict
-            }
-    data = data['data']
-    for t, handle in display_handlers.iteritems():
-        if (isinstance(data, t)):
-            return handle(data)
-    return display_string(data)
-
 def run_remote(cmd):
     r = remote.Remote()
     l = local.Local()
@@ -96,6 +72,7 @@ def run_remote(cmd):
             'git': l.git,
             'hg': l.hg,
             'confirm': local.confirm,
+            'call': lambda x: run_command(x.split(), True),
             'echo': lambda x: sys.stdout.write('{0}\n'.format(x)),
             'echo_error': lambda x: sys.stderr.write('{0}\n'.format(x))
             }
@@ -120,12 +97,13 @@ def sign_request(api_key, method, request):
             }
     return headers
 
-def run_command(cmd):
+def run_command(cmd, internal=False):
     data = None
     cfg = config.load_user_config()
     url = utils.parse_url(cfg.url)
     req = httplib.HTTPConnection(url['host'], int(url['port'] or 80))
-    query = '{0}run?q={1}'.format(url['path'], '%20'.join(cmd))
+    cmd = '%20'.join([urllib.quote(c) if c.find(' ') < 0 else '"{0}"'.format(urllib.quote(c)) for c in cmd])
+    query = '{0}run?{1}={2}'.format(url['path'], 'c' if internal is True else 'q', cmd)
     headers = sign_request(cfg.apikey, 'GET', query)
     headers.update({
         'User-Agent': 'dotcloud/cli'
@@ -140,7 +118,8 @@ def run_command(cmd):
         data = json.loads(data)
         if data['type'] == 'cmd':
             return run_remote(data['data'])
-        display(data)
+        if 'data' in data:
+            print data['data']
     except socket.error, e:
         utils.die('Cannot reach DotCloud service ("{0}").\nPlease check the connectivity and try again.'.format(str(e)))
     except Exception, e:
@@ -150,6 +129,8 @@ def run_command(cmd):
             req.close()
         except Exception:
             pass
+    if internal is True:
+        return
     # Return proper exit code to the shell
     if not data:
         sys.exit(1)
